package com.lee.papervision
{
	import org.papervision3d.core.geom.renderables.Vertex3D;
	import org.papervision3d.objects.DisplayObject3D;
	
	/**
	 * Static utility class for 3d calculations. Most of 
	 * these are already in the PV3D library, as it turns out, 
	 * either via Number3D or Plane3D, but whatever.
	*/
	public class Util3D
	{
		public static var SMALL_NUM:Number = 0.00000001;
		private static var DEGREE:Number = Math.PI/180;
		
		public static function add(v:Vertex3D, w:Vertex3D):Vertex3D
		{
			return new Vertex3D(v.x + w.x, v.y + w.y, v.z + w.z);
		}

		public static function subtract(v:Vertex3D, w:Vertex3D):Vertex3D
		{
			var v:Vertex3D = new Vertex3D(v.x - w.x, v.y - w.y, v.z - w.z);
			return v;
		}

		public static function multiply(v:Vertex3D, n:Number):Vertex3D
		{
			return new Vertex3D(v.x * n, v.y * n, v.z * n);
		}
		
		public static function cross(v:Vertex3D, w:Vertex3D):Vertex3D
		{
			return new Vertex3D(
				 v.y*w.z - v.z*w.y,
				 v.z*w.x - v.x*w.z,
				 v.x*w.y - v.y*w.x
			);
		}

		public static function dot(v:Vertex3D, w:Vertex3D):Number
		{
			return (v.x * w.x + v.y * w.y + w.z * v.z);

		}
		
		public static function lengthSquared(v:Vertex3D):Number
		{
			return v.x*v.x + v.y*v.y + v.z*v.z;
		}
		public static function length(v:Vertex3D):Number
		{
			return Math.sqrt( lengthSquared(v) );
		}
		
		public static function distance(v1:Vertex3D, v2:Vertex3D):Number
		{
			var v:Vertex3D = subtract(v2, v1);
			return length(v);
		}
		
		public static function rotX(v:Vertex3D, theta:Number):Vertex3D
		{
			var y:Number = Math.cos(theta) * v.y - Math.sin(theta) * v.z;
			var z:Number = Math.cos(theta) * v.z + Math.sin(theta) * v.y;
			var x:Number = v.x;
			return new Vertex3D(x, y, z);
		}
		
		public static function rotY(v:Vertex3D, theta:Number):Vertex3D
		{
			var x:Number = Math.cos(theta) * v.x - Math.sin(theta) * v.z;
			var z:Number = Math.cos(theta) * v.z + Math.sin(theta) * v.x;
			var y:Number = v.y;
			return new Vertex3D(x, y, z);
		}
		
		public static function rotZ(v:Vertex3D, theta:Number):Vertex3D
		{
			var x:Number = Math.cos(theta) * v.x - Math.sin(theta) * v.y;
			var y:Number = Math.cos(theta) * v.y + Math.sin(theta) * v.x;
			var z:Number = v.z;
			return new Vertex3D(x, y, z);
		}
		
		public static function getPos(o:DisplayObject3D):Vertex3D
		{
			return new Vertex3D(o.x, o.y, o.z);
		}
		
		public static function setPos(o:DisplayObject3D, v:Vertex3D):void
		{
			o.x = v.x; o.y = v.y; o.z = v.z;
		}
		
		public static function setPosition(o:DisplayObject3D, x:Number, y:Number, z:Number):void
		{
			o.x = x; o.y = y; o.z = z;
		}
		
		public static function tracev(v:Vertex3D):void
		{
			if (!v) { trace('(Vertex3D is undefined)'); return; }
			trace( "x:", int(v.x*1000)/1000 + ",", "y:",int(v.y*1000)/1000 + ",", "z:",int(v.z*1000)/1000 );
		}
			
		/**
		 * Return the normal from 3 vertices.
		 * I.e., the cross product of two edges
		*/
		public static function normal(v0:Vertex3D, v1:Vertex3D, v2:Vertex3D):Vertex3D
		{
			var edgeA:Vertex3D = subtract(v0,v1);
			var edgeB:Vertex3D = subtract(v2,v1);
			
			var v:Vertex3D = cross(edgeA, edgeB);
			return normalize(v);
		}
		
		public static function normalize(v:Vertex3D):Vertex3D
		{
			var l:Number = length(v);
			return new Vertex3D( v.x / l, v.y / l, v.z / l);
		}

		/**
		 * Rotate a 3d point around an arbitrary axis.
		 * Adapted from: http://local.wasp.uwa.edu.au/~pbourke/geometry/rotate/source.c
		 * 
		 * @param	p		Point to be rotated
		 * @param	axis	Axis around which to rotate point, represented as a 3d vector
		 * @param	theta	Angle of rotation
		*/
		public static function rotateArbitraryAxis(p:Vertex3D, axis:Vertex3D, theta:Number):Vertex3D
		{
		   //XYZ q = {0.0,0.0,0.0};
		   var q:Vertex3D = new Vertex3D();
		   //double costheta,sintheta;
		   var costheta:Number, sintheta:Number;
		
		   //Normalise(&r);
		   axis = normalize(axis);
		   costheta = Math.cos(theta);
		   sintheta = Math.sin(theta);
		
		   q.x += (costheta + (1 - costheta) * axis.x * axis.x) * p.x;
		   q.x += ((1 - costheta) * axis.x * axis.y - axis.z * sintheta) * p.y;
		   q.x += ((1 - costheta) * axis.x * axis.z + axis.y * sintheta) * p.z;
		
		   q.y += ((1 - costheta) * axis.x * axis.y + axis.z * sintheta) * p.x;
		   q.y += (costheta + (1 - costheta) * axis.y * axis.y) * p.y;
		   q.y += ((1 - costheta) * axis.y * axis.z - axis.x * sintheta) * p.z;
		
		   q.z += ((1 - costheta) * axis.x * axis.z - axis.y * sintheta) * p.x;
		   q.z += ((1 - costheta) * axis.y * axis.z + axis.x * sintheta) * p.y;
		   q.z += (costheta + (1 - costheta) * axis.z * axis.z) * p.z;
		
		   return(q);
		}



		/**
		 * Adapted to AS from this example:
		 * http://support.realsoftware.com/listarchives/realbasic-games/2005-08/msg00081.html
		 * which was converted from a C example 
		*/
		public static function lineTriangleIntersect(line0:Vertex3D, line1:Vertex3D, v0:Vertex3D, v1:Vertex3D, v2:Vertex3D, treatAsPlane:Boolean=false):Vertex3D
		{
			// vectors
			var rayDirection:Vertex3D, rayPos:Vertex3D, triangleEdgeU:Vertex3D, triangleEdgeV:Vertex3D, planeNormal:Vertex3D, intersection:Vertex3D;
			
			// params to calc ray-plane intersect
			
			// Get triangle edge vectors and plane normal
			triangleEdgeU = Util3D.subtract(v1, v0);
			triangleEdgeV = Util3D.subtract(v2, v0);
			planeNormal = Util3D.cross(triangleEdgeU, triangleEdgeV) // cross product
			
			if (Util3D.lengthSquared(planeNormal)==0) return null; // triangle is degenerate
		
			rayDirection = Util3D.subtract(line1,line0); // ray direction vector
			rayPos = Util3D.subtract(line0,v0);
			
			var negN:Vertex3D = Util3D.multiply(planeNormal, -1);
			var a:Number = Util3D.dot(negN, rayPos);
			var b:Number = Util3D.dot(planeNormal, rayDirection);
			
			// Line is parallel to triangle plane
			if (Math.abs(b) < SMALL_NUM) {
				if (a == 0)	return null // exactly parallel
				else		return null 
			}
		
			// Intersect point of ray with triangle plane
			var r:Number = a / b;
			if (r < 0.0) {
				// trace('? ray goes away from triagle');
				return null;
			}
			if (r > 1.0) {
				// line segment does not intersect?? dunno...
				// return null;
			}
		
			// Intersect point of ray and plane
			intersection = Util3D.add( line0, ( Util3D.multiply(rayDirection,r) ) ); 

			// Is I inside T? ...
			var uu:Number = Util3D.dot(triangleEdgeU,triangleEdgeU);
			var uv:Number = Util3D.dot(triangleEdgeU,triangleEdgeV);
			var vv:Number = Util3D.dot(triangleEdgeV,triangleEdgeV);
			var w:Vertex3D = Util3D.subtract(intersection,v0);
			var wu:Number = Util3D.dot(w,triangleEdgeU);
			var wv:Number = Util3D.dot(w,triangleEdgeV);
			var d:Number = uv * uv - uu * vv;
		
			// Get and test parametric coords
			var s:Number = (uv * wv - vv * wu) / d
			if (s < -SMALL_NUM || s > 1.0 + SMALL_NUM) {
				//trace('? doesnt intersect (a)', s);
				if (!treatAsPlane) return null;
			}
			var t:Number = (uv * wu - uu * wv) / d
			if (t < -SMALL_NUM || (s + t) > 1.0 + SMALL_NUM) {
				//trace('? doesnt intersect (b)', t, (s+t));
				if (!treatAsPlane) return null;
			}
			
			// Is the intersection within the line segment or beyond it?
			if (distance(line0, intersection) > distance(line0, line1)) {
				// trace('beyond',distance(line0, intersection),distance(line0, line1));
				return null;
			}
			
			return intersection // I is in T
		}
		
	}
}